/*
 * lcd.c
 *
 *  Created on: 15/01/2016
 *      Author: eduardo
 */


#include "msp430.h"
#include "lcd.h"

#define MAX_VALUE 	999999
#define MAX_DIGITS	6

uint8_t LCD_Int2BCD(uint32_t value, uint8_t * bcd_array);

#define pos1 9      /* Digit A1 begins at S18 */
#define pos2 5      /* Digit A2 begins at S10 */
#define pos3 3      /* Digit A3 begins at S6  */
#define pos4 18     /* Digit A4 begins at S36 */
#define pos5 14     /* Digit A5 begins at S28 */
#define pos6 7      /* Digit A6 begins at S14 */

// MAPPING for launchpad LCD
const unsigned char lcd_num[10] = {
		0xFC,        // 0
		0x60,        // 1
		0xDB,        // 2
		0xF3,        // 3
		0x67,        // 4
		0xB7,        // 5
		0xBF,        // 6
		0xE4,        // 7
		0xFF,        // 8
		0xF7,        // 9
};

uint8_t LCD_Int2BCD(uint32_t value, uint8_t * bcd_array)
{
	uint8_t i;

	for(i = 0; i < MAX_DIGITS; i++)
	{
		bcd_array[i] = 0;
	}

	if(value > MAX_VALUE)
	{
		return 0;
	}

	while(value > 0)
	{
		if(value >= 100000)
		{
			value -= 100000;
			bcd_array[5] += 1;
			continue;
		}

		if(value >= 10000)
		{
			value -= 10000;
			bcd_array[4] += 1;
			continue;
		}

		if(value >= 1000)
		{
			value -= 1000;
			bcd_array[3] += 1;
			continue;
		}

		if(value >= 100)
		{
			value -= 100;
			bcd_array[2] += 1;
			continue;
		}

		if(value >= 10)
		{
			value -= 10;
			bcd_array[1] += 1;
			continue;
		}

		if(value > 0)
		{
			value -= 1;
			bcd_array[0] += 1;
		}
	}

	return 1;
}

void LCD_Init(void)
{
	// Initialize LCD segments 0 - 21; 26 - 43
	LCDCPCTL0 = 0xFFFF;
	LCDCPCTL1 = 0xFC3F;
	LCDCPCTL2 = 0x0FFF;

	// Initialize LCD_C
	// ACLK, Divider = 1, Pre-divider = 16; 4-pin MUX
	LCDCCTL0 = LCDDIV__1 | LCDPRE__16 | LCD4MUX | LCDLP;

	// VLCD generated internally,
	// V2-V4 generated internally, v5 to ground
	// Set VLCD voltage to 2.60v
	// Enable charge pump and select internal reference for it
	LCDCVCTL = VLCD_1 | VLCDREF_0 | LCDCPEN;

	LCDCCPCTL = LCDCPCLKSYNC;               // Clock synchronization enabled

	LCDCMEMCTL = LCDCLRM;                   // Clear LCD memory

	//Turn LCD on
	LCDCCTL0 |= LCDON;

}

uint8_t LCD_WriteNum(uint32_t value)
{
	uint8_t bcd_array[MAX_DIGITS];

	if(value > MAX_VALUE)
	{
		return 0;
	}

	if (LCD_Int2BCD(value, bcd_array) == 0)
	{
		return 0;
	}

	LCDMEM[pos6] = lcd_num[bcd_array[0]];
	LCDMEM[pos5] = lcd_num[bcd_array[1]];
	LCDMEM[pos4] = lcd_num[bcd_array[2]];
	LCDMEM[pos3] = lcd_num[bcd_array[3]];
	LCDMEM[pos2] = lcd_num[bcd_array[4]];
	LCDMEM[pos1] = lcd_num[bcd_array[5]];

	return 1;
}
